unit uWebServiceCorreios;

interface

uses
  uWSDL_Correios, Classes;

type
  { Adaptee }
  TWebServiceCorreios = class
  private
    // variveis para armazenar os objetos solicitados na comunicao do WebService
    Parametros: consultaCEP;
    Resposta: enderecoERP;

    // mtodo para sobrescrever a requisio SOAP
    procedure BeforeExecute(const MethodName: string; SOAPRequest: TStream);
  public
    destructor Destroy; override;

    // cria o objeto da classe de parmetros da consulta e o preenche com o CEP informado
    procedure DefinirParametrosConsulta(const CEP: string);

    // envia uma requisio ao WebService para consultar o endereo
    procedure ConsultarCEP;

    // retorna o contedo das propridades do objeto da classe "enderecoERP"
    function ObterResposta(const Informacao: string): string;
  end;

implementation

uses
  SysUtils, SOAPHTTPClient;

{ TWebServiceCorreios }

procedure TWebServiceCorreios.BeforeExecute(const MethodName: string;
  SOAPRequest: TStream);
var
  Request: TStringList;
begin
  // sobrescreve a requisio SOAP para deix-la de acordo com a estrutura
  // exigida pelo WebService

  Request := TStringList.Create;
  try
    Request.Text :=
      '<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:cli="http://cliente.bean.master.sigep.bsb.correios.com.br/">' +
      '  <soapenv:Header/>' +
      '    <soapenv:Body>' +
      '      <cli:consultaCEP>' +
      '        <cep>' + Parametros.cep + '</cep>' +
      '      </cli:consultaCEP>' +
      '    </soapenv:Body>' +
      '</soapenv:Envelope>';

    SOAPRequest.Position := 0;
    SOAPRequest.Size := Length(Request.Text);
    Request.SaveToStream(SOAPRequest);
  finally
    FreeAndNil(Request);
  end;
end;

procedure TWebServiceCorreios.ConsultarCEP;
var
  Correios: AtendeCliente;
  HTTPRIO: THTTPRIO;
begin
  // instancia o objeto da classe THTTPRIO
  HTTPRIO := THTTPRIO.Create(nil);

  // associa o evento "OnBeforeExecute" para sobrescrever a requisio SOAP
  HTTPRIO.OnBeforeExecute := BeforeExecute;

  // obtm uma instncia do WSDL
  Correios := GetAtendeCliente(True, '', HTTPRIO);

  // chama o mtodo do WebService para consultar o CEP e receber o retorno
  Resposta := Correios.consultaCEP(Parametros).return;
end;

procedure TWebServiceCorreios.DefinirParametrosConsulta(const CEP: string);
begin
  // cria o objeto da classe "consultaCEP" e preenche a propriedade "cep"
  Parametros := consultaCEP.Create;
  Parametros.cep := CEP;
end;

destructor TWebServiceCorreios.Destroy;
begin
  FreeAndNil(Parametros);
  inherited;
end;

function TWebServiceCorreios.ObterResposta(const Informacao: string): string;
begin
  // retorna o contedo da propriedade do objeto "FoResposta"
  // conforme o tipo de informao solicitado no parmetro
  
  result := EmptyStr;

  if Informacao = 'Logradouro' then
    result := Resposta.end_
  else if Informacao = 'Bairro' then
    result := Resposta.bairro
  else if Informacao = 'Cidade' then
    result := Resposta.cidade;
end;

end.
